declare type EmptyArray = [];
export declare type TransactionCallPayload = {
    from: string;
    to: string;
    gas?: string;
    gasPrice?: string;
    maxPriorityFeePerGas?: string;
    maxFeePerGas?: string;
    value?: string;
    data?: string;
};
export declare type TransactionPayload = {
    from: string;
    to?: string;
    gas?: string;
    gasPrice?: string;
    maxPriorityFeePerGas?: string;
    maxFeePerGas?: string;
    value?: string;
    data?: string;
    nonce?: string;
};
export declare type Transaction = {
    accessList?: string[];
    blockHash: string;
    blockNumber: string;
    chainId?: string;
    from: string;
    gas?: string;
    gasLimit?: string;
    gasPrice?: string;
    hash: string;
    maxPriorityFeePerGas?: string;
    maxFeePerGas?: string;
    nonce: string;
    r: string;
    s: string;
    to?: string | null;
    transactionIndex: string;
    type: string;
    v: string;
    value: string;
    input: string;
};
export declare type PendingTransaction = {
    accessList?: string[];
    blockHash?: null;
    blockNumber?: null;
    chainId?: string;
    from: string;
    gas?: string;
    gasPrice?: string;
    gasLimit?: string;
    hash: string;
    maxPriorityFeePerGas?: string;
    maxFeePerGas?: string;
    nonce: string;
    r: string;
    s: string;
    to?: string | null;
    transactionIndex: string;
    type: string;
    v: string;
    value: string;
    input: string;
};
export declare type Block = {
    number: string;
    hash: string;
    parentHash: string;
    nonce: string;
    sha3Uncle: string;
    logsBloom: string;
    transactionsRoot: string;
    stateRoot: string;
    receiptsRoot: string;
    miner: string;
    difficulty: string;
    totalDifficulty: string;
    extraData: string;
    size: string;
    gasLimit: string;
    gasUsed: string;
    timestamp: string;
    transactions: string[];
    uncles: string[];
};
export declare type PendingBlock = {
    number?: null;
    hash?: string;
    parentHash: string;
    nonce?: null;
    sha3Uncle: string;
    logsBloom?: null;
    transactionsRoot: string;
    stateRoot: string;
    receiptsRoot: string;
    miner: string;
    difficulty: string;
    totalDifficulty: string;
    extraData: string;
    size: string;
    gasLimit: string;
    gasUsed: string;
    timestamp: string;
    transactions: string[];
    uncles: string[];
};
export declare type TransactionReceipt = {
    blockHash: string;
    blockNumber: string;
    contractAddress?: string | null;
    cumulativeGasUsed: string;
    effectiveGasPrice: string;
    from: string;
    gasUsed: string;
    logs: Log[];
    logsBloom: string;
    root?: string;
    status?: "0x0" | "0x1";
    to?: string | null;
    transactionHash: string;
    transactionIndex: string;
    type: string;
    confirmations?: string;
    byzantium?: boolean;
};
export declare type FilterObject = {
    address?: string;
    fromBlock?: string;
    toBlock?: string;
    topics?: string[];
    blockHash?: string;
};
export declare type Log = {
    removed: boolean;
    logIndex: string;
    transactionIndex: string;
    transactionHash: string;
    blockHash: string;
    blockNumber: string;
    address: string;
    data: string;
    topics: string[];
};
export declare type PendingLog = {
    removed: boolean;
    logIndex?: null;
    transactionIndex?: null;
    transactionHash?: null;
    blockHash?: null;
    blockNumber?: null;
    address: string;
    data: string;
    topics: string[];
};
declare type EthAccounts = {
    method: "eth_accounts";
    params: EmptyArray;
    response: string[];
};
declare type EthBlockNumber = {
    method: "eth_blockNumber";
    params: EmptyArray;
    response: string;
};
declare type EthCall = {
    method: "eth_call";
    params: [callPayload: TransactionCallPayload, blockTag: string];
    response: unknown;
};
declare type EthChainId = {
    method: "eth_chainId";
    params: EmptyArray;
    response: string;
};
declare type EthCoinbase = {
    method: "eth_coinbase";
    params: unknown;
    response: unknown;
};
declare type EthEstimateGas = {
    method: "eth_estimateGas";
    params: [callPayload: TransactionCallPayload];
    response: string;
};
declare type EthFeeHistory = {
    method: "eth_feeHistory";
    params: [string, string, []];
    response: {
        baseFeePerGas: string[];
        gasUsedRatio: string[];
        reward?: string[];
        oldestBlock: string;
    };
};
declare type EthGetBalance = {
    method: "eth_getBalance";
    params: [address: string, blockTag: string];
    response: string;
};
declare type EthGasPrice = {
    method: "eth_gasPrice";
    params: EmptyArray;
    response: string;
};
declare type EthGetBlockByHash = {
    method: "eth_getBlockByHash";
    params: [blockHash: string, showTransactionDetailsFlag: boolean];
    response: Block | PendingBlock | null;
};
declare type EthGetBlockByNumber = {
    method: "eth_getBlockByNumber";
    params: [blockTag: string, showTransactionDetailsFlag: boolean];
    response: Block | PendingBlock | null;
};
declare type EthGetBlockTransactionCountByHash = {
    method: "eth_getBlockTransactionCountByHash";
    params: [blockHash: string];
    response: string;
};
declare type EthGetBlockTransactionCountByNumber = {
    method: "eth_getBlockTransactionCountByNumber";
    params: [blockTag: string];
    response: string;
};
declare type EthGetCode = {
    method: "eth_getCode";
    params: [address: string, blockTag: string];
    response: string;
};
declare type EthGetLogs = {
    method: "eth_getLogs";
    params: [filterObject: FilterObject];
    response: Log[] | PendingLog[];
};
declare type EthGetStorageAt = {
    method: "eth_getStorageAt";
    params: [address: string, storagePosition: string, blockTag: string];
    response: string;
};
declare type EthGetTransactionByBlockHashAndIndex = {
    method: "eth_getTransactionByBlockHashAndIndex";
    params: [blockHash: string, transactionIndex: string];
    response: Transaction | PendingTransaction | null;
};
declare type EthGetTransactionByBlockNumberAndIndex = {
    method: "eth_getTransactionByBlockNumberAndIndex";
    params: [blockTag: string, transactionIndex: string];
    response: Transaction | PendingTransaction | null;
};
declare type EthGetTransactionByHash = {
    method: "eth_getTransactionByHash";
    params: [transactionHash: string];
    response: Transaction | PendingTransaction | null;
};
declare type EthGetTransactionCount = {
    method: "eth_getTransactionCount";
    params: [address: string, blockTag: string];
    response: string;
};
declare type EthGetTransactionReceipt = {
    method: "eth_getTransactionReceipt";
    params: [transactionHash: string];
    response: TransactionReceipt | null;
};
declare type EthGetUncleByBlockHashAndIndex = {
    method: "eth_getUncleByBlockHashAndIndex";
    params: [blockHash: string, uncleIndexPosition: string];
    response: Block | null;
};
declare type EthGetUncleByBlockNumberAndIndex = {
    method: "eth_getUncleByBlockNumberAndIndex";
    params: [blockTag: string, uncleIndexPosition: string];
    response: Block | null;
};
declare type EthGetUncleCountByBlockHash = {
    method: "eth_getUncleCountByBlockHash";
    params: [blockHash: string];
    response: string;
};
declare type EthGetUncleCountByBlockNumber = {
    method: "eth_getUncleCountByBlockNumber";
    params: [blockTag: string];
    response: string;
};
declare type EthGetWork = {
    method: "eth_getWork";
    params: EmptyArray;
    response: string[];
};
declare type EthMining = {
    method: "eth_mining";
    params: EmptyArray;
    response: boolean;
};
declare type EthHashrate = {
    method: "eth_hashrate";
    params: EmptyArray;
    response: string;
};
declare type EthProtocolVersion = {
    method: "eth_protocolVersion";
    params: EmptyArray;
    response: string;
};
declare type EthSendRawTransaction = {
    method: "eth_sendRawTransaction";
    params: [transactionData: string];
    response: string;
};
declare type EthSendTransaction = {
    method: "eth_sendTransaction";
    params: [transactionPayload: TransactionPayload];
    response: string;
};
declare type EthSign = {
    method: "eth_sign";
    params: [address: string, data: string];
    response: string;
};
declare type EthSignTransaction = {
    method: "eth_signTransaction";
    params: [transationPayload: TransactionPayload];
    response: string;
};
declare type EthSubmitWork = {
    method: "eth_submitWork";
    params: [string, string, string];
    response: boolean;
};
declare type EthSyncing = {
    method: "eth_syncing";
    params: EmptyArray;
    response: unknown;
};
declare type NetListening = {
    method: "net_listening";
    params: EmptyArray;
    response: boolean;
};
declare type NetPeerCount = {
    method: "net_peerCount";
    params: EmptyArray;
    response: string;
};
declare type NetVersion = {
    method: "net_version";
    params: EmptyArray;
    response: string;
};
declare type EthSubscribe = {
    method: "eth_subscribe";
    params: ["newHeads"] | ["logs", {
        address?: string | string[];
        topics?: string[];
    }] | ["newPendingTransactions"] | ["syncing"];
    response: string;
};
declare type EthUnsubscribe = {
    method: "eth_unsubscribe";
    params: [subscriptionId: string];
    response: boolean;
};
declare type EthPersonalSign = {
    method: "eth_personalSign";
    params: unknown[];
    response: string;
};
declare type EthSignTypedData = {
    method: "eth_signTypedData";
    params: [
        address: string,
        typedData: {
            types?: unknown;
            primaryType?: string;
            domain?: unknown;
            message?: unknown;
        }
    ];
    response: string;
};
declare type EthRequestAccounts = {
    method: "eth_requestAccounts";
    params: EmptyArray;
    response: string[];
};
export declare type JsonRPCMethod = EthAccounts | EthBlockNumber | EthCall | EthChainId | EthCoinbase | EthEstimateGas | EthFeeHistory | EthGetBalance | EthGasPrice | EthGetBlockByHash | EthGetBlockByNumber | EthGetBlockTransactionCountByHash | EthGetBlockTransactionCountByNumber | EthGetCode | EthGetLogs | EthGetStorageAt | EthGetTransactionByBlockHashAndIndex | EthGetTransactionByBlockNumberAndIndex | EthGetTransactionByHash | EthGetTransactionCount | EthGetTransactionReceipt | EthGetUncleByBlockHashAndIndex | EthGetUncleByBlockNumberAndIndex | EthGetUncleCountByBlockHash | EthGetUncleCountByBlockNumber | EthGetWork | EthMining | EthHashrate | EthProtocolVersion | EthSendRawTransaction | EthSendTransaction | EthSign | EthSubmitWork | EthSyncing | NetListening | NetPeerCount | NetVersion | EthSubscribe | EthUnsubscribe | EthSignTransaction | EthPersonalSign | EthSignTypedData | EthRequestAccounts;
export declare type JsonRPCMethodName = JsonRPCMethod["method"];
export {};
