"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.generateTestingUtils = void 0;
var providers_1 = require("./providers");
var testing_utils_1 = require("./testing-utils");
var defaultGenerationOptions = {
    providerType: "default",
    verbose: false,
};
/**
 * Generate the testing utils associated with a mock provider
 * @param options.providerType Type of the provider to mock, default to `default`
 * @param options.verbose If true, the JSON-RPC request will be logged
 * @returns The testing utils for one provider
 */
function generateTestingUtils(_a) {
    var _b = _a === void 0 ? defaultGenerationOptions : _a, providerType = _b.providerType, verbose = _b.verbose;
    var provider = providerType === "MetaMask"
        ? new providers_1.MetaMaskProvider({ verbose: verbose })
        : providerType === "WalletConnect"
            ? new providers_1.WalletConnectProvider({ verbose: verbose })
            : new providers_1.Provider({ verbose: verbose });
    var testingUtils = new testing_utils_1.TestingUtils(provider);
    return testingUtils;
}
exports.generateTestingUtils = generateTestingUtils;
